#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            oumanSendMessage.py
#
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
#
HELP_TEXT = """
# DISCLAIMER:      You can use this software to configure your controller 
#                  equipment. The author of this software is not in any way
#                  responsible for possible configuration errors or the 
#                  parameter values this software writes to the controller. 
#                  The software user must read thoroughly this document,
#                  be sure to enter valid configuration values and accept 
#                  the fact that the software author is not responsible 
#                  of your configuration. USE THIS SOFTWARE AT YOUR OWN RISK.
#
# Usage:           python oumanSendMessage.py --help
#                  python oumanSendMessage.py [-v] [--nowait] <serialDev> [<message>]
#
#                  Options (options override the configuration file):
#                    --help      - Prints out a help and exits.
#
#                    -v          - Enable debug/verbose logging.
#
#                    --nowait    - Do not wait for returning message.
#
#                  Parameters:
#                    serialDev   - Path to serial device (eg. /dev/ttyS0)
#
#                    message     - Message text (if omitted, try to read a message from the controller)
#
#                  Examples:
#                    [taloLogger]% python ./oumanSendMessage.py /dev/ttyS0 "EH01 KELLONAIKA"
#                    EH01 KELLONAIKA: 30.09.2010 15:49
#                    [taloLogger]%   
#
"""
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.0b
#
# Date:            09.02.2011
#
# Functions:       -
#                   
# Description:     Script to send a SMS like message to Ouman controller
#                  using the direct serial connection to Ouman.
#
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.4)
# 
#                  For Ouman controller communications:
#                    Python Serial Port Extension: pySerial  
#                    (http://pyserial.wiki.sourceforge.net/pySerial)
#
# Version history: ** 30.09.2010 v1.0a (Olli Lammi) **
#                  First beta version. 
#
#                  ** 90.02.2011 v1.0b (Olli Lammi) **
#                  Removed device type parameter. 
#
###########################################################################

# Imports

import sys, os 
import string  

import configuration
import oumanSerial
import log


####### main ##########################################################

def main():
    verbose = 0
    nowait = 0
    serial_port = ''
    message = ''
    
    # handle command line parameters
    i = 1
    j = 0
    while i < len(sys.argv):
        if sys.argv[i] == '--help':
            Help()
            sys.exit(1)
        elif sys.argv[i] == '-v':
            verbose = 1
        elif sys.argv[i] == '--nowait':
            nowait = 1
        elif j == 0:
            serial_port = sys.argv[i]
            j = j + 1
        elif j >= 1:
            if len(message) > 0:
                message = message + ' '
            message = message + sys.argv[i]
            j = j + 1
        else:
            Usage()
            sys.exit(1)

        i = i + 1
        
    if len(serial_port) <= 0:
        Usage()
        sys.exit(1) 


    conf = configuration.Configuration()
    conf.addConfigurable(log.Logger)
    conf.setValue('CONSOLE_LOGGING', 'true')
    if verbose:
        conf.setValue('VERBOSE_LOGGING', 'true')
    LOG = log.Logger(conf)
    log.Logging.setLogger(LOG)

    oser = oumanSerial.OumanSerial(serial_port, 'DEBUG')
       
    if len(message) > 160:
        LOG.log("ERROR: Message lenght exeeds 160 characters.")
        sys.exit(1)   
       
    if len(message) > 0:        
        res = oser.runQueryId(97)
        if len(res) <= 0:
            LOG.log("ERROR: Cannot get data from Ouman controller.")
            sys.exit(1)
        res = oumanSerial.getOumanDataPart(res)[2:]
        if res != '\xFF':
            LOG.log("ERROR: Invalid state from Ouman controller.")
            print len(res)
            print res
            sys.exit(1)
        
        # generate message data from 
        msgdata = '123456' + ('\x00' * 15)
        msgdata = msgdata + message
        msgdata = msgdata + ('\x00' * (161-len(message)))

        resdata = oser.runWriteId(98, msgdata)
        if len(resdata) <= 0:
            LOG.log("ERROR: Cannot get response data from Ouman controller.")
            sys.exit(1)
            
        # check that the response contains correct id
        resid = oumanSerial.getOumanDataPart(resdata)
        if len(resid) < 2 or (ord(resid[0]) << 8) + ord(resid[1]) != 98:
            LOG.log("ERROR: Invalid response data from Ouman controller.")
            sys.exit(1)
        
    if nowait:
        sys.exit(1)

    stat = 0
    num = 0
    while not stat and num < 10:
        num = num + 1
        res = oser.runQueryId(45)
        if len(res) <= 0:
            LOG.log("ERROR: Cannot get data from Ouman controller.")
            sys.exit(1)
        res = oumanSerial.getOumanDataPart(res)[2:]
        if ord(res[0]) & 16 != 0:
            stat = 1

    if not stat:
        LOG.log("ERROR: Ouman controller not responding to message.")
        sys.exit(1)

    while stat:
        res = oser.runQueryId(99)
        if len(res) <= 0:
            LOG.log("ERROR: Cannot get data from Ouman controller.")
            sys.exit(1)
        res = oumanSerial.getOumanDataPart(res)[2:]
        tstr = ''
        for i in range(len(res)-21):
            if res[i+21] == '\x00':
                break
            tstr = tstr + res[i+21]
        print tstr

        res = oser.runWriteId(55, '\xFE')
        res = oser.runWriteId(96, '\xFF')

        res = oser.runQueryId(45)
        if len(res) <= 0:
            LOG.log("ERROR: Cannot get data from Ouman controller.")
            sys.exit(1)
        res = oumanSerial.getOumanDataPart(res)[2:]
        if ord(res[0]) & 16 == 0:
            stat = 0
  
  
def Usage():
    print 'USAGE:  python oumanSendMessage.py --help'
    print '        python oumanSendMessage.py [-v] [--nowait] <serialDev> <oumanDevice> [<message>]'


def Help():
    temps = string.split(HELP_TEXT, '\n')
    for temp in temps:
        if len(temp) > 0 and temp[0] == '#':
            temp = temp[1:]
        if len(temp) > 0 and temp[0] == ' ':
            temp = temp[1:]
        print temp
 

###########################################################################

if __name__ == "__main__":
  main()
