#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            regoSerial.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         0.1b
#
# Date:            16.11.2010
#
# Description:     Library to communicate with the Rego controllers.
#                  
# Supported devices: 
#                  Rego600 series controllers.
#
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.3)
# 
#                  Python Serial Port Extension: pySerial  
#                  (http://pyserial.wiki.sourceforge.net/pySerial)
#
# Version history: ** 16.11.2010 v0.1a (Olli Lammi) **
#                  First test version. 
#
#                  ** 16.11.2010 v0.1b (Olli Lammi) **
#                  Minor bug corrections. 
#
###########################################################################

# Imports

import sys, os 
import string

import log
import lockfile
import binary

# pySerial module, not loaded until init of RegoSerial class instance
serial = None

###########################################################################

# Constants

BAUDRATE = 19200 
TIMEOUT = 2

###########################################################################

# Device configuration data

TYPE_TEMP10 = 1
TYPE_INT16 = 2 


REGO_DEVICES = { 'REGO600':  [ ['GT1 Radiator return', 521, TYPE_TEMP10], \
                               ['GT2 Outdoor', 522, TYPE_TEMP10], \
                               ['GT3 Hot water', 523, TYPE_TEMP10], \
                               ['GT4 Forward', 524, TYPE_TEMP10], \
                               ['GT5 Room', 525, TYPE_TEMP10], \
                               ['GT6 Hot gas', 526, TYPE_TEMP10], \
                               ['GT8 Heat fluid out', 527, TYPE_TEMP10], \
                               ['GT9 Heat fluid in', 528, TYPE_TEMP10], \
                               ['GT10 Cold fluid in', 529, TYPE_TEMP10], \
                               ['GT11 Cold fluid out', 530, TYPE_TEMP10], \
                               ['GT3x External hot water', 531, TYPE_TEMP10], \
                               ['P3 Cold fluid pump', 509, TYPE_INT16], \
                               ['Compressor', 510, TYPE_INT16], \
                               ['Additional heat 1', 511, TYPE_INT16], \
                               ['Additional heat 2', 512, TYPE_INT16], \
                               ['P1 Radiator pump', 515, TYPE_INT16], \
                               ['P2 Heat fluid pump', 516, TYPE_INT16], \
                               ['Three-way valve', 517, TYPE_INT16], \
                               ['Alarm', 518, TYPE_INT16] \
                             ] \
               }


###########################################################################

# Classes

class RegoSerial(log.Logging):
    def __init__(self, port, device):
        log.Logging.__init__(self, 'RegoSerial')
        
        global serial
        serial = __import__('serial') 
        
        self.SERPORT = port
        self.REGO_DEVICE = device
        self.serio = 0
          
        self.LOCK = lockfile.LockFile(self.SERPORT)
    
    def openPort(self):
        if not REGO_DEVICES.has_key(self.REGO_DEVICE):
            self.Log("ERROR: Invalid REGO device type: " + self.REGO_DEVICE)
            return 0

        if not self.LOCK.lock():
            self.Log("ERROR: Unable to aquire lockfile for regoSerial port: " + self.LOCK.getName())
            return 0

        try:
            self.serio = serial.Serial(port=self.SERPORT,baudrate=BAUDRATE, \
                                bytesize=serial.EIGHTBITS, \
                                parity=serial.PARITY_NONE, stopbits=serial.STOPBITS_ONE, \
                                xonxoff=0, rtscts=0, dsrdtr=0, timeout=TIMEOUT)
            if not self.serio.isOpen():
                self.serio = 0
                self.LOCK.free()
                return 0 
        except:
            self.serio = 0
            self.LOCK.free()
            return 0
        
        self.Debug("Opened regoSerial port: " + self.SERPORT)
        return 1
        
    def closePort(self):
        if self.serio:
            try:
                self.serio.close()
            except:
                pass
            self.serio = 0
        self.LOCK.free()
        self.Debug("Closed regoSerial port: " + self.SERPORT)

    def isOpen(self):
        if self.serio and self.serio.isOpen():
            return 1
        return 0

    def runQueryCommand(self, cmd):
        id = 0
        type = 0
        for t in REGO_DEVICES[self.REGO_DEVICE]:
            if t[0] == cmd:
                id = t[1]
                type = t[2]
                break

        if type == 0:
            self.Log("ERROR: Invalid REGO key: " + cmd)
            return ""

        res = self.runQueryId(id)
        if len(res) > 0:
            return convertRegoMessage(type, res)
        return ""

    def runQueryId(self, id):
        self.Debug("Running regoSerial query with id " + repr(id))
        if not self.isOpen():
            if not self.openPort():
                self.Log("ERROR: Cannot open serial port.")
                return ""
            (stat, res) = serialQuery(self.serio, id)
            self.closePort()
        else:
            (stat, res) = serialQuery(self.serio, id)
        self.Debug("Received rego response: " + repr(res))
        
        if stat == 1:
            return res
        elif stat == -1:
            self.Log("ERROR: Timeout reading message from Rego")
            return ""
        elif stat == -2:
            self.Log("ERROR: Error in return message from Rego")
            return ""

    def getVersion(self):
        self.Debug("Getting regoSerial version info.")
        if not self.isOpen():
            if not self.openPort():
                self.Log("ERROR: Cannot open serial port.")
                return ""
            (stat, res) = rawSerialQuery(self.serio, 0x7F, 0, 0, 5)
            self.closePort()
        else:
            (stat, res) = rawSerialQuery(self.serio, 0x7F, 0, 0, 5)
        self.Debug("Received rego response: " + repr(res))
        
        if stat == 1:
            return convertRegoMessage(TYPE_INT16, res)
        elif stat == -1:
            self.Log("ERROR: Timeout reading version from Rego")
            return ""
        elif stat == -2:
            self.Log("ERROR: Error in return version from Rego")
            return ""
        

###########################################################################

# Functions

# returns (status, response)
# status: 1 = ok, -1 = timeout, -2 = error in response
def serialQuery(serio, id):
    return rawSerialQuery(serio, 2, id, 0, 5)
    
    
# returns (status, response)
# status: 1 = ok, -1 = timeout, -2 = error in response
def rawSerialQuery(serio, cmd, id, value, l):
    sendbuff = '\x81' + chr(cmd)
    temp1 = binary.int16ToSevenBitBuff(id)
    temp2 = binary.int16ToSevenBitBuff(value)
    crc = 0
    for i in temp1 + temp2:
        sendbuff = sendbuff + chr(i)
        crc = crc ^ i
    sendbuff = sendbuff + chr(crc)

    serio.write(sendbuff)
    serio.flush()

    res = ""
    temp = " "
    while len(temp) > 0:
        temp = serio.read()
        if len(temp) <= 0:
            if checkRegoMessage(res, l) == 1:
                return (1, res)
            else:
                return (-1, '')
        res = res + temp
        stat = checkRegoMessage(res, l)
        if stat == 1:
            return (1, res)
        elif stat == 2:
            return (-2, '')


# returns int
# 0 = ok, but not ready
# 1 = ok
# 2 = error
def checkRegoMessage(data, l):
    if len(data) <= 0:
        return 0

    # first byte is 0x01
    if len(data) >= 1 and data[0] != chr(0x01):
        return 2

    # next len-2 bytes are data

    if len(data) < l:
        return 0

    # if package is already longer than it should
    if len(data) > l:
        return 2

    # calculate CRC
    crc = 0
    for i in range(1,l-1):    
        crc = crc ^ ord(data[i])
    if crc != ord(data[l-1]):
        return 2
    
    return 1


def getRegoDataPart(data, l):
    if checkRegoMessage(data, l) != 1:
        return ""

    return data[1:l-1]


def convertRegoMessage(type, data):
    # get data part
    data = getRegoDataPart(data, 5)

    if len(data) != 3:
        return ""

    if type == TYPE_INT16:
        temp = binary.sevenBitBuffToInt16(data)
        temp = "%d" % temp
        return temp
    elif type == TYPE_TEMP10:
        temp = binary.sevenBitBuffToInt16(data)
        temp = "%.1f" % (temp / 10.0)
        return temp

    return ""


