#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            sourceOneWire.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.5c
#
# Date:            24.03.2013
#
# Description:     TaloLogger configuration wrapper for OneWire library.
#                  
# Version history: ** 11.10.2011 v1.4e (Olli Lammi) **
#                  Moved from main taloLogger application module. 
#
#                  ** 13.02.2012 v1.4i (Olli Lammi) **
#                  Multisupport.
#
#                  ** 20.01.2013 v1.5b (Olli Lammi) **
#                  Support for EDS OWSERVER-ENET-2.
#
#                  ** 24.03.2013 v1.5c (Olli Lammi) **
#                  Support for OWFS init, pre and post commands.
#
###########################################################################

# Imports

import os, string

import configuration
import dataSource
import oneWire


###########################################################################

# Classes

class OneWireFSConf(oneWire.OneWireFS, configuration.Configurable, dataSource.DataSource):
    def __init__(self, conf, modname):
        configuration.Configurable.__init__(self)
        self.setModuleName(modname)
        oneWire.OneWireFS.__init__(self)
        dataSource.DataSource.__init__(self)
        self.setID(modname)
        self.handleConfiguration(conf)
        self.setTimeout(self.dsTimeout)
        self.setUseExternalProcess(self.useexternalprocess)

    def getServiceName(self):
        return self.getModuleName()

    def handleConfiguration(self, conf):
        self.useexternalprocess = conf.isTrue('USE_EXTERNAL_PROCESS', self.getModuleName())
        self.setDevices( self.handleOWFSConf(conf) )
        return (1, '')
        
    def initConfiguration(self):
        # run init commands
        if not runCommands(self.INIT_COMMANDS):
            return (0, 'Error running OWFS init commands.')
        return (1, '')

    def handleOWFSConf(self, conf):
        self.INIT_COMMANDS = []
        self.PRE_COMMANDS = []
        self.POST_COMMANDS = []
        
        for item in conf.getValue('INITCMD', [], self.getModuleName()):
            self.INIT_COMMANDS.append(string.strip(item))
        for item in conf.getValue('PRECMD', [], self.getModuleName()):
            self.PRE_COMMANDS.append(string.strip(item))
        for item in conf.getValue('POSTCMD', [], self.getModuleName()):
            self.POST_COMMANDS.append(string.strip(item))
                
        devmap = {}
        for item in conf.getValue('OWFS', [], self.getModuleName()):
            temps = string.split(item, ':', 2)
            if len(temps) < 3:
                self.log("ERROR: Invalid OWFS configuration: " + item)
            else:
                try:
                    key = string.strip(temps[0])
                    type = int(string.strip(temps[1]))
                    value = string.strip(temps[2])
                except:
                    type = 0
                if len(key) <= 0 or len(value) <= 0 or type <= 0:
                    self.log("ERROR: Invalid OWFS configuration: " + item)
                else:
                    devmap[key] = [type, value]
        return devmap

    def runDataSourceQueryCommandImpl(self, cmds):
        if not runCommands(self.PRE_COMMANDS):
            self.log("ERROR: Error running OWFS pre measure commands.")
        temp = self.runDataSourceQueryMultiWrapper(cmds)
        if not runCommands(self.POST_COMMANDS):
            self.log("ERROR: Error running OWFS post measure commands.")
        return temp

    @staticmethod
    def getAllowedConfigurationKeys():
        return (['USE_EXTERNAL_PROCESS'], ['OWFS', 'INITCMD', 'PRECMD', 'POSTCMD'])


class OneWireDigiTempConf(oneWire.OneWireDigiTemp, configuration.Configurable, dataSource.DataSource):
    def __init__(self, conf, modname):
        configuration.Configurable.__init__(self)
        self.setModuleName(modname)
        oneWire.OneWireDigiTemp.__init__(self, '', '', '')
        dataSource.DataSource.__init__(self)
        self.setID(modname)
        self.handleConfiguration(conf)
        self.setTimeout(self.dsTimeout)
        
    def getServiceName(self):
        return self.getModuleName()

    def handleConfiguration(self, conf): 
        self.DT_BINARY = conf.getValue('LOCATION', '', self.getModuleName())
        self.DT_RC = conf.getValue('CONFFILE', '', self.getModuleName())
        self.DT_PORT = conf.getValue('PORT', '', self.getModuleName())
        return (1, '')
        
    def initConfiguration(self):
        # check the existence of the digitemp executable
        if not os.access(self.DT_BINARY, os.X_OK):
            return (0, 'OneWireDigiTemp: Unable to find or execute DigiTemp executable: ' + self.DT_BINARY)
        if not os.access(self.DT_RC, os.R_OK):
            return (0, 'OneWireDigiTemp: Unable to access DigiTemp configuration file: ' + self.DT_RC)
        if len(self.DT_PORT) <= 0:
            return (-1, 'OneWireDigiTemp: Unspecified DigiTemp port.')
        return (1, '')

    def runDataSourceQueryCommandImpl(self, cmds):
        return self.runDataSourceQueryMultiWrapper(cmds)

    @staticmethod
    def getAllowedConfigurationKeys():
        return (['LOCATION', 'PORT', 'CONFFILE'], [])


class OneWireEDSOWSERVERConf(oneWire.OneWireEDSOWSERVER, configuration.Configurable, dataSource.DataSource):
    def __init__(self, conf, modname):
        configuration.Configurable.__init__(self)
        self.setModuleName(modname)
        oneWire.OneWireEDSOWSERVER.__init__(self)
        dataSource.DataSource.__init__(self)
        self.setID(modname)
        self.handleConfiguration(conf)
        
    def getServiceName(self):
        return self.getModuleName()

    def handleConfiguration(self, conf): 
        self.URL = conf.getValue('URL', '', self.getModuleName())
        self.setDatapoints(self.handleEDSOWSERVERConf(conf))
        return (1, '')
        
    def handleEDSOWSERVERConf(self, conf):
        pointmap = {}
        for item in conf.getValue('DATAPOINT', [], self.getModuleName()):
            temps = string.split(item, ':', 4)
            if len(temps) < 4:
                self.log("ERROR: Invalid DATAPOINT configuration: " + item)
            else:
                try:
                    key = string.strip(temps[0])
                    romid = string.upper(string.strip(temps[1]))
                    valtype = int(string.strip(temps[2]))
                    valkey = string.upper(string.strip(temps[3]))
                except:
                    key = ''
                if len(key) <= 0:
                    self.log("ERROR: Invalid DATAPOINT configuration: " + item)
                else:
                    pointmap[key] = [romid, valtype, valkey]
        return pointmap
        
    def initConfiguration(self):
        if len(self.URL) <= 0:
            return (-1, 'Missing EDS OWSERVER-ENET-2 URL.')
        if len(self.DATAPOINTS.keys()) <= 0:
            return (0, 'No data points configured for EDSOWSERVER datasource module.')
        return (1, '')

    def runDataSourceQueryCommandImpl(self, cmds):
        return self.runQueryCommand(cmds)

    @staticmethod
    def getAllowedConfigurationKeys():
        return (['URL'], ['DATAPOINT'])


###########################################################################

# Functions

def runCommands(cmds):
    status = True
    for cmd in cmds:
        temp = string.strip(cmd)
        if len(temp) <= 0:
            continue
        try:
            os.system(temp)
        except:
            status = False
            
    return status
