#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            thermIQSerial.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         0.1b
#
# Date:            26.09.2013
#
# Description:     Library to communicate with the ThermIQ interface module.
#                  (see www.thermiq.net)
#                  
# Supported devices: 
#                  ThermIQ interface module (www.thermiq.net)
#
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.3)
# 
#                  Python Serial Port Extension: pySerial  
#                  (http://pyserial.wiki.sourceforge.net/pySerial)
#
# Version history: ** 18.10.2011 v0.1a (Olli Lammi) **
#                  First test version. 
#
#                  ** 26.09.2013 v0.1b (Olli Lammi) **
#                  Increased serial timeout value. 
#
###########################################################################

# Imports

import sys, os 
import string

import log
import lockfile
import binary

# pySerial module, not loaded until init of ThermIQSerial class instance
serial = None

###########################################################################

# Constants

BAUDRATE = 9600 
TIMEOUT = 10


###########################################################################

# Device configuration data

TYPE_INT16 = 1
TYPE_UINT16 = 2
TYPE_INT16_10 = 3
TYPE_UINT16_10 = 4

THERMIQ_DEVICES = { 'DEFAULT':  [ ['outdoor temp', 0, TYPE_INT16], \
                                  ['room temp', 1, TYPE_INT16], \
                                  ['room temp decimal', 2, TYPE_INT16_10], \
                                  ['room target temp', 3, TYPE_INT16], \
                                  ['room target temp decimal', 4, TYPE_INT16_10], \
                                  ['supply line temp', 5, TYPE_INT16], \
                                  ['return line temp', 6, TYPE_INT16], \
                                  ['hot water temp', 7, TYPE_INT16], \
                                  ['heat source supply temp', 8, TYPE_INT16], \
                                  ['heat source return temp', 9, TYPE_INT16], \
                                  ['cooling temp', 10, TYPE_INT16], \
                                  ['supplyline temp shunt', 11, TYPE_INT16], \
                                  ['electrical current', 12, TYPE_INT16], \
                                  ['aux heater status', 13, TYPE_UINT16], \
                                  ['supply line target temp', 14, TYPE_INT16], \
                                  ['supply line target temp shunt', 15, TYPE_INT16], \
                                  ['status', 16, TYPE_UINT16], \
                                  ['status2', 17, TYPE_UINT16], \
                                  ['status3', 18, TYPE_UINT16], \
                                  ['alarm', 19, TYPE_UINT16], \
                                  ['alarm2', 20, TYPE_UINT16], \
                                  ['demand1', 21, TYPE_UINT16], \
                                  ['demand2', 22, TYPE_UINT16], \
                                  ['pressurepipe temp', 23, TYPE_INT16], \
                                  ['hot water supplyline temp', 24, TYPE_INT16], \
                                  ['integral', 25, TYPE_INT16], \
                                  ['integral reached alimit', 26, TYPE_INT16], \
                                  ['status4', 27, TYPE_UINT16], \
                                  ['minimun time to start', 28, TYPE_UINT16], \
                                  ['program version', 29, TYPE_UINT16], \
                                  ['flowline pump speed', 30, TYPE_UINT16], \
                                  ['heat source pump speed', 31, TYPE_UINT16], \
                                  ['status5', 32, TYPE_UINT16], \
                                  ['indoor target temp', 50, TYPE_INT16], \
                                  ['mode', 51, TYPE_UINT16], \
                                  ['curve', 52, TYPE_INT16], \
                                  ['curve min', 53, TYPE_INT16], \
                                  ['curve max', 54, TYPE_INT16], \
                                  ['curve +5', 55, TYPE_INT16], \
                                  ['curve 0', 56, TYPE_INT16], \
                                  ['curve -5', 57, TYPE_INT16], \
                                  ['heatstop', 58, TYPE_INT16], \
                                  ['lowered room temp', 59, TYPE_INT16], \
                                  ['room factor', 60, TYPE_INT16], \
                                  ['curve2', 61, TYPE_INT16], \
                                  ['curve2 min', 62, TYPE_INT16], \
                                  ['curve2 max', 63, TYPE_INT16], \
                                  ['status6', 64, TYPE_UINT16], \
                                  ['status7', 65, TYPE_UINT16], \
                                  ['status8', 66, TYPE_UINT16], \
                                  ['pressurepipe temp limit', 67, TYPE_INT16], \
                                  ['start temperature hot water', 68, TYPE_INT16], \
                                  ['hot water operating time', 69, TYPE_UINT16], \
                                  ['heat pump operating time', 70, TYPE_UINT16], \
                                  ['legionella interval', 71, TYPE_UINT16], \
                                  ['legionella stop temp', 72, TYPE_INT16], \
                                  ['integral limit A1', 73, TYPE_INT16], \
                                  ['hysteresis heat pump', 74, TYPE_INT16], \
                                  ['return line temp max limit', 75, TYPE_INT16], \
                                  ['minimum starting interval', 76, TYPE_UINT16], \
                                  ['heat source temp min limit', 77, TYPE_INT16], \
                                  ['cooling target', 78, TYPE_INT16], \
                                  ['integral limit A2', 79, TYPE_INT16], \
                                  ['hysteresis limit aux', 80, TYPE_INT16], \
                                  ['max step aux', 81, TYPE_UINT16], \
                                  ['electrical current max limit', 82, TYPE_UINT16], \
                                  ['shunt time', 83, TYPE_UINT16], \
                                  ['hot water stop temp', 84, TYPE_INT16], \
                                  ['manual test mode', 85, TYPE_UINT16], \
                                  ['status9', 86, TYPE_UINT16], \
                                  ['language', 87, TYPE_UINT16], \
                                  ['status10', 88, TYPE_UINT16], \
                                  ['factory settings', 89, TYPE_UINT16], \
                                  ['reset runtime counters', 90, TYPE_UINT16], \
                                  ['calibration outdoor sensor', 91, TYPE_INT16], \
                                  ['calibration supplyline sensor', 92, TYPE_INT16], \
                                  ['calibration returnline sensor', 93, TYPE_INT16], \
                                  ['calibration hot water sensor', 94, TYPE_INT16], \
                                  ['calibration heat source out sensor', 95, TYPE_INT16], \
                                  ['calibration heat source in sensor', 96, TYPE_INT16], \
                                  ['heating system type', 97, TYPE_UINT16], \
                                  ['addon status', 98, TYPE_UINT16], \
                                  ['logging time', 99, TYPE_UINT16], \
                                  ['heat source runout duration', 100, TYPE_UINT16], \
                                  ['heat source runin duration', 101, TYPE_UINT16], \
                                  ['legionella peak heating enable', 102, TYPE_UINT16], \
                                  ['legionella peak heating duration', 103, TYPE_UINT16], \
                                  ['runtime compressor', 104, TYPE_UINT16], \
                                  ['status11', 105, TYPE_UINT16], \
                                  ['runtime a-heat1', 106, TYPE_UINT16], \
                                  ['status12', 107, TYPE_UINT16], \
                                  ['runtime hot water', 108, TYPE_UINT16], \
                                  ['status13', 109, TYPE_UINT16], \
                                  ['runtime passive cooling', 110, TYPE_UINT16], \
                                  ['status14', 111, TYPE_UINT16], \
                                  ['runtime active cooling', 112, TYPE_UINT16], \
                                  ['status15', 113, TYPE_UINT16], \
                                  ['runtime a-heat2', 114, TYPE_UINT16], \
                                  ['status16', 115, TYPE_UINT16], \
                                  ['status17', 116, TYPE_UINT16]
                                ] \
                  }

STATUS_MASK_HEAT_SOURCE_PUMP = 1 
STATUS_MASK_COMPRESSOR = 2 
STATUS_MASK_RADIATOR_PUMP = 4 
STATUS_MASK_HOTWATER_PRODUCTION = 8
STATUS_MASK_AUX_2 = 16
STATUS_MASK_SHUNT_MINUS = 32
STATUS_MASK_SHUNT_PLUS = 64 
STATUS_MASK_AUX_1 = 128

STATUS2_MASK_SHUNTGROUP_PLUS = 1
STATUS2_MASK_SHUNTGROUP_MINUS = 2
STATUS2_MASK_SHUNT_COOLING_MINUS = 4
STATUS2_MASK_SHUNT_COOLING_PLUS = 8
STATUS2_MASK_ACTIVE_COOLING = 16
STATUS2_MASK_PASSIVE_COOLING = 32
STATUS2_MASK_ALARM = 64

ALARM_MASK_HIGHPR_PRESSOSTATE = 1
ALARM_MASK_LOWPR_PRESSOSTATE = 2
ALARM_MASK_MOTORCIRCUIT_BREAKER = 4
ALARM_MASK_LOW_TEMP_HEAT_SOURCE = 8
ALARM_MASK_LOW_FLOW_HEAT_SOURCE = 16

ALARM2_MASK_OUTDOOR_T_SENSOR = 1
ALARM2_MASK_SUPPLYLINE_T_SENSOR = 2
ALARM2_MASK_RETURNLINE_T_SENSOR = 4
ALARM2_MASK_HOTWATER_T_SENSOR = 8
ALARM2_MASK_INDOOR_T_SENSOR = 16
ALARM2_MASK_OVERHEATING = 32
ALARM2_MASK_INCORRECT_3_PHASE_ORDER = 64

STATUS_AUX_HEATER_MASK_3_KW = 1
STATUS_AUX_HEATER_MASK_6_KW = 2

STATUS_ADDON_MASK_PHASE_ORDER_MEASUREMENT = 1
STATUS_ADDON_MASK_HGW = 2
STATUS_ADDON_MASK_OPTIMUM = 4
STATUS_ADDON_MASK_FLOW_GUARD = 8


###########################################################################

# Classes

class ThermIQSerial(log.Logging):
    def __init__(self, port, device):
        log.Logging.__init__(self, 'ThermIQSerial')
        
        global serial
        serial = __import__('serial') 
        
        self.SERPORT = port
        self.THERMIQ_DEVICE = device
        self.serio = 0
          
        self.LOCK = lockfile.LockFile(self.SERPORT)
    
    def openPort(self):
        if not THERMIQ_DEVICES.has_key(self.THERMIQ_DEVICE):
            self.Log("ERROR: Invalid THERMIQ device type: " + self.THERMIQ_DEVICE)
            return 0

        if not self.LOCK.lock():
            self.Log("ERROR: Unable to aquire lockfile for thermIQSerial port: " + self.LOCK.getName())
            return 0

        try:
            self.serio = serial.Serial(port=self.SERPORT,baudrate=BAUDRATE, \
                                bytesize=serial.EIGHTBITS, \
                                parity=serial.PARITY_NONE, stopbits=serial.STOPBITS_ONE, \
                                xonxoff=0, rtscts=0, dsrdtr=0, timeout=TIMEOUT)
            if not self.serio.isOpen():
                self.serio = 0
                self.LOCK.free()
                return 0 
        except:
            self.serio = 0
            self.LOCK.free()
            return 0
        
        self.Debug("Opened thermIQSerial port: " + self.SERPORT)
        return 1
        
    def closePort(self):
        if self.serio:
            try:
                self.serio.close()
            except:
                pass
            self.serio = 0
        self.LOCK.free()
        self.Debug("Closed thermIQSerial port: " + self.SERPORT)

    def isOpen(self):
        if self.serio and self.serio.isOpen():
            return 1
        return 0

    def runQueryCommand(self, cmd):
        id = 0
        type = 0
        for t in THERMIQ_DEVICES[self.THERMIQ_DEVICE]:
            if t[0] == cmd:
                id = t[1]
                type = t[2]
                break

        if type == 0:
            self.Log("ERROR: Invalid THERMIQ key: " + cmd)
            return ""

        res = self.runQueryId(id)
        if len(res) > 0:
            return convertThermIQMessage(type, res, id)
        return ""

    def runQueryId(self, id):
        self.Debug("Running thermIQSerial query with id " + repr(id))
        if not self.isOpen():
            if not self.openPort():
                self.Log("ERROR: Cannot open serial port.")
                return ""
            (stat, res) = serialQuery(self.serio, id)
            self.closePort()
        else:
            (stat, res) = serialQuery(self.serio, id)
        self.Debug("Received thermIQ response: " + repr(res))
        
        if stat == 1:
            return res
        elif stat == -1:
            self.Log("ERROR: Timeout reading message from ThermIQ")
            return ""
        elif stat == -2:
            self.Log("ERROR: Error in return message from ThermIQ")
            return ""

    def getVersion(self):
        self.Debug("Getting thermIQSerial version info.")
        if not self.isOpen():
            if not self.openPort():
                self.Log("ERROR: Cannot open serial port.")
                return ""
            (stat, res) = rawSerialQuery(self.serio, 'ati\n')
            self.closePort()
        else:
            (stat, res) = rawSerialQuery(self.serio, 'ati\n')
        self.Debug("Received thermIQ response: " + repr(res))
        
        if stat == 1:
            return res
        elif stat == -1:
            self.Log("ERROR: Timeout reading version from ThermIQ")
            return ""
        elif stat == -2:
            self.Log("ERROR: Error in return version from ThermIQ")
            return ""
        

###########################################################################

# Functions

# returns (status, response)
# status: 1 = ok, -1 = timeout, -2 = error in response
def serialQuery(serio, id):
    (stat, res) = sendCommand(serio, 'atr%02x\n' % id, 'OK\n')

    if stat == 0:
        return (-1, '')

    (stat, data) = checkThermIQMessage(res, id)
    if stat != 1:
        return (-2, '')

    return (1, data)


def sendCommand(ser, command, waitfor):
    ser.write(command)
    ser.flush()
    res = ""
    temp = " "
    while len(temp) > 0:
        temp = ser.read()
        if len(temp) <= 0:
            ser.flushInput()
            return (0, "")
        res = res + temp
        str = res.replace('\r\n','\n').replace('\r','\n')
        if str.endswith(waitfor):
            ser.flushInput()
            return (1, str)
    ser.flushInput()
    return (0, "")


# returns (status, response)
# status: 1 = ok, -1 = timeout, -2 = error in response
def rawSerialQuery(serio, query):
    (stat, res) = sendCommand(serio, query, 'OK\n')

    if stat == 0:
        return (-1, '')

    if len(res) <= 0:
        return (-2, '')

    return (1, res)



# returns (int, value)
# 0 = ok, but not ready
# 1 = ok
# 2 = error
def checkThermIQMessage(data, id):
    if len(data) <= 0:
        return (0, '')

    rows = string.split(data, '\n')
    for row in rows:
        row = string.strip(row)
        if len(row) == 7 and row[2] == '=':
            vid = -1
            try:
                vid = int(row[:2], 16)
            except:
                vid = -1
            if vid == id:
                return (1, row[3:])

    return (2, '')


def convertThermIQMessage(type, data, id):
    if len(data) < 4:
        return ""

    temp = 0
    try:
        temp = int(data, 16)
    except:
        return ""

    if type == TYPE_INT16:
        temp = binary.twosComplementToInt(temp, 16)
        temp = "%d" % temp
        return temp
    elif type == TYPE_UINT16:
        temp = "%d" % temp
        return temp
    elif type == TYPE_INT16_10:
        temp = binary.twosComplementToInt(temp, 16)
        temp = "%.1f" % (temp / 10.0)
        return temp
    elif type == TYPE_UINT16_10:
        temp = "%.1f" % (temp / 10.0)
        return temp

    return ""


