#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            thermiaSerial.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         0.1b
#
# Date:            04.09.2010
#
# Description:     Library to communicate with the Thermia pump.
#                  
# Supported devices: 
#                  ???
#
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.3)
# 
#                  Python Serial Port Extension: pySerial  
#                  (http://pyserial.wiki.sourceforge.net/pySerial)
#
# Version history: ** 29.04.2010 v0.1a (Olli Lammi) **
#                  First test version. 
#
#                  ** 04.09.2010 v0.1b (Olli Lammi) **
#                  Corrections to bugs found in beta testing. 
#
###########################################################################

# Imports

import sys, os 
import string

import log
import lockfile
import binary

# pySerial module, not loaded until init of ThermiaSerial class instance
serial = None

###########################################################################

# Constants

BAUDRATE = 9600 
TIMEOUT = 2

###########################################################################

# Device configuration data

TYPE_INT16 = 1 
TYPE_UINT16 = 2

THERMIA_DEVICES = { 'DEFAULT':  [ ['outdoor', 0, TYPE_INT16, []], \
                                  ['room temp', 1, TYPE_INT16, []], \
                                  ['room target temp', 3, TYPE_INT16, []], \
                                  ['supply line temp', 5, TYPE_INT16, []], \
                                  ['return line temp', 6, TYPE_INT16, []], \
                                  ['hot water temp', 7, TYPE_INT16, []], \
                                  ['heat source supply temp', 8, TYPE_INT16, []], \
                                  ['heat source return temp', 9, TYPE_INT16, []], \
                                  ['calc shunt flow value', 14, TYPE_INT16, []], \
                                  ['status', 16, TYPE_UINT16, [0, 0]], \
                                  ['integral', 25, TYPE_INT16, []], \
                                  ['heat curve', 52, TYPE_INT16, []], \
                                  ['curve min', 53, TYPE_INT16, []], \
                                  ['curve max', 54, TYPE_INT16, []], \
                                  ['curve +5', 55, TYPE_INT16, []], \
                                  ['curve', 56, TYPE_INT16, []], \
                                  ['curve -5', 57, TYPE_INT16, []], \
                                  ['heatstop', 58, TYPE_INT16, []], \
                                  ['lowered room temp', 59, TYPE_INT16, []], \
                                  ['start temperature hot water', 68, TYPE_INT16, []], \
                                  ['integral A1 start HP', 73, TYPE_INT16, []], \
                                  ['stop temperature hot water', 84, TYPE_INT16, []], \
                                  ['runtime heatpump', 104, TYPE_UINT16, []], \
                                  ['runtime a-heat1', 106, TYPE_UINT16, []], \
                                  ['runtime hot water', 108, TYPE_UINT16, []], \
                                  ['runtime a-heat2', 114, TYPE_UINT16, []], \
                                ] \
                  }







STATE_MASK_GROUND_LOOP_PUMP = 1 
STATE_MASK_COMPRESSOR = 2 
STATE_MASK_RADIATOR_PUMP = 4 
STATE_MASK_THREE_WAY_VALVE = 8 


###########################################################################

# Classes

class ThermiaSerial(log.Logging):
    def __init__(self, port, device):
        log.Logging.__init__(self, 'ThermiaSerial')
        
        global serial
        serial = __import__('serial') 
        
        self.SERPORT = port
        self.THERMIA_DEVICE = device
        self.serio = 0
          
        self.LOCK = lockfile.LockFile(self.SERPORT)
    
    def openPort(self):
        if not THERMIA_DEVICES.has_key(self.THERMIA_DEVICE):
            self.Log("ERROR: Invalid THERMIA device type: " + self.THERMIA_DEVICE)
            return 0

        if not self.LOCK.lock():
            self.Log("ERROR: Unable to aquire lockfile for thermiaSerial port: " + self.LOCK.getName())
            return 0

        try:
            self.serio = serial.Serial(port=self.SERPORT,baudrate=BAUDRATE, \
                                bytesize=serial.EIGHTBITS, \
                                parity=serial.PARITY_NONE, stopbits=serial.STOPBITS_ONE, \
                                xonxoff=0, rtscts=0, dsrdtr=0, timeout=TIMEOUT)
            if not self.serio.isOpen():
                self.serio = 0
                self.LOCK.free()
                return 0 
        except:
            self.serio = 0
            self.LOCK.free()
            return 0
        
        self.Debug("Opened thermiaSerial port: " + self.SERPORT)
        return 1
        
    def closePort(self):
        if self.serio:
            try:
                self.serio.close()
            except:
                pass
            self.serio = 0
        self.LOCK.free()
        self.Debug("Closed thermiaSerial port: " + self.SERPORT)

    def isOpen(self):
        if self.serio and self.serio.isOpen():
            return 1
        return 0

    def runQueryCommand(self, cmd):
        id = 0
        type = 0
        params = []
        for t in THERMIA_DEVICES[self.THERMIA_DEVICE]:
            if t[0] == cmd:
                id = t[1]
                type = t[2]
                params = t[3]
                break

        if type == 0:
            self.Log("ERROR: Invalid THERMIA key: " + cmd)
            return ""

        res = self.runQueryId(id, params)
        if len(res) > 0:
            return convertThermiaMessage(type, res, id)
        return ""

    def runQueryId(self, id, params):
        self.Debug("Running thermiaSerial query with id " + repr(id))
        if not self.isOpen():
            if not self.openPort():
                self.Log("ERROR: Cannot open serial port.")
                return ""
            (stat, res) = serialQuery(self.serio, id, params)
            self.closePort()
        else:
            (stat, res) = serialQuery(self.serio, id, params)
        self.Debug("Received thermia response: " + repr(res))
        
        if stat == 1:
            return res
        elif stat == -1:
            self.Log("ERROR: Timeout reading message from Thermia")
            return ""
        elif stat == -2:
            self.Log("ERROR: Error in return message from Thermia")
            return ""

    def getVersion(self):
        self.Debug("Getting thermiaSerial version info.")
        if not self.isOpen():
            if not self.openPort():
                self.Log("ERROR: Cannot open serial port.")
                return ""
            (stat, res) = rawSerialQuery(self.serio, [0x76], 9)
            self.closePort()
        else:
            (stat, res) = rawSerialQuery(self.serio, [0x76], 9)
        self.Debug("Received thermia response: " + repr(res))
        
        if stat == 1:
            return res[2:]
        elif stat == -1:
            self.Log("ERROR: Timeout reading version from Thermia")
            return ""
        elif stat == -2:
            self.Log("ERROR: Error in return version from Thermia")
            return ""
        

###########################################################################

# Functions

# returns (status, response)
# status: 1 = ok, -1 = timeout, -2 = error in response
def serialQuery(serio, id, params):
    sendbuff = '\x74' + chr(id)
    for i in params:
        sendbuff = sendbuff + chr(i)

    serio.write(sendbuff)
    serio.flush()

    res = ""
    temp = " "
    while len(temp) > 0:
        temp = serio.read()
        if len(temp) <= 0:
            if checkThermiaMessage(res, id) == 1:
                return (1, res)
            else:
                return (-1, '')
        res = res + temp
        stat = checkThermiaMessage(res, id)
        if stat == 1:
            return (1, res)
        elif stat == 2:
            return (-2, '')


# returns (status, response)
# status: 1 = ok, -1 = timeout, -2 = error in response
def rawSerialQuery(serio, query, rlen):
    sendbuff = ''
    for ch in query:
        sendbuff = sendbuff + chr(ch)
    
    serio.write(sendbuff)
    serio.flush()

    res = ""
    temp = " "
    while len(temp) > 0:
        temp = serio.read()
        if len(temp) <= 0:
            if len(res) == rlen:
                return (1, res)
            else:
                return (-1, '')
        res = res + temp
        if len(res) == rlen:
            stat = 1
        else:
            stat = 0
        if stat == 1:
            return (1, res)
        elif stat == 2:
            return (-2, '')


# returns int
# 0 = ok, but not ready
# 1 = ok
# 2 = error
def checkThermiaMessage(data, id):
    if len(data) <= 0:
        return 0

    # first byte is 0x54
    if len(data) >= 1 and data[0] != chr(0x54):
        return 2

    # second byte is 0x5C
    if len(data) >= 2 and data[1] != chr(0x5C):
        return 2

    # third byte is id
    if len(data) >= 3 and data[2] != chr(id):
        return 2

    # if package is already longer than it should
    if len(data) > 5:
        return 2

    if len(data) < 5:
        return 0

    return 1


def getThermiaDataPart(data, id):
    if checkThermiaMessage(data, id) != 1:
        return ""

    return data[3:5]


def convertThermiaMessage(type, data, id):
    # get data part and remove id part
    data = getThermiaDataPart(data, id)

    if len(data) < 2:
        return ""

    if type == TYPE_INT16:
        temp = 0
        temp = (ord(data[1]) << 8) + ord(data[0])
        temp = binary.twosComplementToInt(temp, 16)
        temp = "%d" % temp
        return temp
    elif type == TYPE_UINT16:
        temp = (ord(data[1]) << 8) + ord(data[0])
        temp = "%d" % temp
        return temp

    return ""


