#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            threads.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.1a
#
# Date:            06.01.2013
#
# Description:     Library to implement threading and locking classes in
#                  python.
#                  
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.3)
# 
# Version history: ** 02.12.2008 v1.0a (Olli Lammi) **
#                  First version. 
#
#                  ** 06.01.2013 v1.1a (Olli Lammi) **
#                  Added termination controlled version. 
#
###########################################################################

# Imports

import sys
import thread, signal, time


###########################################################################

# Classes

class Lock:
    def __init__(self):
        self.mylock = thread.allocate_lock()

    def lock_wait(self):
        return self.mylock.acquire()

    def lock_immediate(self):
        return self.mylock.acquire(0)

    def free(self):
        return self.mylock.release()


RUNNING = 0
TERMINATING = 1
TERMINATED = 2

class Thread:
    def __init__(self):
        self.lock = Lock()
        self.tid = None
        self.thr_state = TERMINATED

    def start(self):
        if self.lock.lock_immediate():
            self.setRunning()
            try:
                self.tid = thread.start_new_thread(self.main, (self,))
            except:
                self.setTerminated()
                self.lock.free()
                return 0
            return 1
        else:
            return 0

    def main(self, starter):
        try:
            starter.run()
        except:
            print "Error running thread (thread id " + `self.tid` + "):", sys.exc_info()[0], sys.exc_info()[1]
            sys.exc_clear()
        starter.setTerminated()
        starter.lock.free()        

    def run(self):
        print "Unimplemented thread run method."

    def wait(self):
        self.lock.lock_wait()
        self.lock.free()

    def wait_sleeping(self):
        while 1:
            if self.lock.lock_immediate():
               self.lock.free()
               return 
            else:
               time.sleep(1)

    def wait_sleeping_timeout(self, timeout, interval = 1.0):
        sttime = time.time()
        while 1:
            if self.lock.lock_immediate():
                self.lock.free()
                return True
            elif timeout > 0 and time.time() - sttime > timeout:
                return False
            else:
                time.sleep(interval)

    def setRunning(self):
        self.thr_state = RUNNING
        
    def setTerminated(self):
        if self.thr_state < 0:
            self.thr_state = -TERMINATED
        else:
            self.thr_state = TERMINATED

    def terminate(self):
        if self.thr_state == RUNNING:
            self.thr_state = TERMINATING
        if self.thr_state < 0:
            self.thr_state = -self.thr_state

    def setFail(self):
        self.thr_state = -TERMINATING

    def isRunning(self):
        if self.thr_state == RUNNING:
            return 1
        return 0

    def isFailed(self):
        if self.thr_state < 0:
            return 1
        return 0

    def isTerminated(self):
        return (self.thr_state != 0)

    def hasTerminated(self):
        return (self.thr_state == TERMINATED or self.thr_state == -TERMINATED)
    
    
